#include "cquad.h"

double cquad_dqagie(double f(), double bound, int inf, double epsabs, double epsrel, double *abserr, int *neval, int *ier, double *alist, double *blist, double *rlist, double *elist, int *iord, int *last)
{
/******************************************************************************
***  Date written:         03/31/2007  (mm/dd/yyyy)                         ***
***-------------------------------------------------------------------------***
***  Last revision date:   11/02/2012  (mm/dd/yyyy)                         ***
***-------------------------------------------------------------------------***
***  Author:    Gianluca Russo    (gianlurusso@fastwebnet.it)               ***
***             Salvatore Cuomo   (salvatore.cuomo@unina.it)                ***
***                                                                         ***
***             University of Naples "Federico II"                          ***
***             Dipartimento di Matematica e Applicazioni "R. Caccioppoli"  ***
***									    ***
*** 									    ***
***									    ***
*** This code is provided "as is", without any warranty of any kind,        ***
*** either expressed or implied, including but not limited to, any implied  ***
*** warranty of merchantibility or fitness for any purpose. In no event     ***
*** will any party who distributed the code be liable for damages or for    ***
*** any claim(s) by any other party, including but not limited to, any      ***
*** lost profits, lost monies, lost data or data rendered inaccurate,	    ***
*** losses sustained by third parties, or any other special, incidental or  ***	
*** consequential damages arising out of the use or inability to use the    ***
*** program, even if the possibility of such damages has been advised	    ***	
*** against. The entire risk as to the quality, the performance, and the    ***	
*** fitness of the program for any particular purpose lies with the party   ***
*** using the code.							    ***
***									    ***
*** This code, and any derivative of this code, may not be used in a        ***
*** commercial package without the prior explicit written permission of     ***
*** the authors. Verbatim copies of this code may be made and distributed   ***
*** in any medium, provided that this copyright notice is not removed or    ***
*** altered in any way. No fees may be charged for distribution of the      ***
*** codes, other than a fee to cover the cost of the media and a            ***
*** reasonable handling fee.						    ***
***-------------------------------------------------------------------------***
***  Purpose:                                                               ***
***                                                                         ***
***    The function provides a C interface to the fortran routine dqagie    ***
***    of quadpack package.                                                 ***
***                                                                         ***
***    The function calculates an approximation result to a given definite  ***
***    integral i = integral of f over (bound, +infinity) or i = integral   ***
***    of f over (-infinity, bound) or i = integral of f over (-infinity,   ***
***    +infinity) hopefully satisfying following claim for accuracy,        ***
***    hopefully satisfying following claim for accuracy:                   ***
***                                                                         ***
***              abs(i - result) <= max(epsabs, epsrel * abs(i))            ***
***-------------------------------------------------------------------------***
***  Description:                                                           ***
***                                                                         ***
***    Handles integration over infinite intervals. The infinite range is   ***
***    mapped onto a finite interval and then the same strategy as in       ***
***    cquad_dqagse is applied.                                             *** 
***                                                                         ***
***    Expert user version (provide more information and control than the   ***
***    easier version cquad_dqagi).                                         ***
***                                                                         ***
***    Double precision version.                                            ***
***-------------------------------------------------------------------------***
***  Input:                                                                 ***
***                                                                         ***
***    f        function subprogram defining the integrand function f(x)    ***
***                                                                         ***
***    bound    finite bound of integration range                           ***
***             (has no meaning if interval is doubly-infinite)             ***
***                                                                         ***
***    inf      indicating the kind of integration range involved:          ***
***                                                                         ***
***                 inf =  1  corresponds to  (bound, +infinity)            ***
***                 inf = -1  corresponds to  (-infinity, bound)            ***
***                 inf =  2  corresponds to  (-infinity, +infinity)        ***
***                                                                         ***
***    epsabs   absolute accoracy requested                                 ***
***                                                                         ***
***    epsrel   relative accoracy requested                                 ***
***-------------------------------------------------------------------------***
***  Output:                                                                ***
***                                                                         ***
***    res      approximation to the integral                               ***
***                                                                         ***
***    abserr   estimate of the modulus of the absolute error               ***
***                                                                         ***
***    neval    number of integrand evaluations                             ***
***                                                                         ***
***    ier      normal termination flag:                                    ***
***                                                                         ***
***               ier = 0 --> normal and reliable termination of the        ***
***                           function. It is assumed that the requested    ***
***                           accuracy has been achieved.                   ***
***                                                                         ***
***               ier = 1 --> maximum number of subdivisions allowed has    ***
***                           been achieved.                                ***
***                                                                         ***
***               ier = 2 --> the occurrence of roundoff error is detected  ***
***                           which prevents the requested tolerance from   ***
***                           being achieved.                               ***
***                           The error may be under-estimated.             ***
***                                                                         ***
***               ier = 3 --> extremely bad integrand behaviour occurs at   ***
***                           some points of the integration interval.      ***
***                                                                         ***
***               ier = 4 --> the algorithm does not converge. Roundoff     ***
***                           error is detected in the extrapolation table. ***
***                           It is presumed that the requested tolerance   ***
***                           cannot be achieved, and that the returned     ***
***                           result is the best which can be obtained.     ***
***                                                                         ***
***               ier = 5 --> the integral is probably divergent, or slowly ***
***                           convergent. it must be noted that divergence  ***
***                           can occur with any other value of ier > 0.    ***
***                                                                         ***
***               ier = 6 --> the input is invalid because epsabs <= 0 and  ***
***                           epsrel < max (50 * rel.mach.acc., 0.5d-28).   ***
***                           res, abserr, neval, last, rlist(1), elist(1)  ***
***                           and iord(1) are set to zero. alist(1) and     ***
***                           blist(1) are set to a and b respectively.     ***
***                                                                         ***
***    alist    vector of dimension at least limit, the first last          ***
***             elements of which are the left end points of the            ***
***             subintervals in the partition of the transformed            ***
***             integration range (0,1)                                     *** 
***                                                                         ***
***    blist    vector of dimension at least limit, the first last          ***
***             elements of which are the right end points of the           ***
***             subintervals in the partition of the transformed            ***
***             integration range (0,1)                                     ***  
***                                                                         ***
***    rlist    vector of dimension at least limit, the first last          ***
***             elements of which are the integral approximations on the    ***
***             subintervals                                                ***
***                                                                         ***
***    elist    vector of dimension at least limit, the first last          ***
***             elements of which are the moduli of the absolute error      ***
***             estimates on the subintervals                               ***
***                                                                         ***
***    last     number of subintervals actually produced in the subdivision ***
***             process                                                     ***
***-------------------------------------------------------------------------***
***  Workspace:                                                             ***
***                                                                         ***
***    limit    gives an upper bound on the number of subintervals in the   ***
***             partition of (a,b) (at least 1)                             ***
***                                                                         ***
***    iord     vector of dimension at least limit, the first k elements    ***
***             of which are pointers to the error estimates over the       ***
***             subintervals, such that elist(iord(1)), ..., elist(iord(k)) ***
***             form a decreasing sequence, with k = last if last <=        ***
***             (limit/2 + 2), and k = limit + 1 - last otherwise           ***
***-------------------------------------------------------------------------***
***  References:      QUADPACK - Piessens, de Doncker, Uberhuber, Kahaner   ***
***                                                                         ***
***                   http://netlib.org                                     ***
******************************************************************************/

    int limit = LIMIT;
    double res;

    dqagie_(f, &bound, &inf, &epsabs, &epsrel, &limit, &res, abserr, neval, ier, alist, blist, rlist, elist, iord, last);

    return res;
}
